import { Output, EventEmitter, AfterViewChecked, OnChanges } from "@angular/core";

export abstract class BaseComponent implements AfterViewChecked, OnChanges {
    /**
     * true après le 1er affichage du composant
     */
    private _firstViewChecked = false;

    /**
     * true si on souhaite qu'un événement soit émis en même temps que l'appel à afterFirstViewChecked()
     */
    protected emitFirstViewCheck = false;

    /**
     * événement émis en même temps que l'appel à afterFirstViewChecked()
     */
    @Output()
    private firstViewCheck: EventEmitter<void>;

    constructor() {
        this.firstViewCheck = new EventEmitter();
    }

    public ngAfterViewChecked() {
        if (!this._firstViewChecked) {
            this._firstViewChecked = true;

            this.afterFirstViewChecked();

            if (this.emitFirstViewCheck) {
                this.firstViewCheck.emit();
            }
        }
    }

    public ngOnChanges() {
    }

    /**
     * appelé une fois, après l'affichage complet du composant
     */
    protected afterFirstViewChecked() {
    }
}
